#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include "sx12.h"

int errordisp = 1;
int verbose = 1;

int errorchk(int flag, char *errmsg)
{
	if(flag && errordisp)
		printf("Error: %s\n",errmsg);

	return flag;
}

main(int argc, char * argv[])
{
    long            BufferSize = 32768L;
    unsigned char   *Buffer;
    union REGS      regs;
    int             fhan;
    int             first = 1;
    int             fcnt =0;
    int             idcnt = 0;
    int             VideoDone = 0;
    X12PARAM _far   *Param;
    unsigned long   oldVQsize = 0;
    unsigned long   oldVQsizeErr = 0;
    clock_t         OldClock;
    unsigned int    buflen;
    int             a;
    char            fname[10];

    if(argc >= 2)
    {
        for(a = 2; a < argc; a++)
		{
            if(strnicmp(argv[a],"/q",2)==0)
			{
                verbose = 0; 
			}

            if(strnicmp(argv[a],"/n",2)==0)
			{
                errordisp = 0; 
			}
	        if(strnicmp(argv[a],"/b",2)==0)
			{
	            sscanf(argv[a]+2,"%ld",&BufferSize);
                if(errorchk(BufferSize > 63 || BufferSize < 1, "Buffer size out of range"))
                    return 0;
				BufferSize *= 1024;
			}
		}
    }
    else
    {
        _splitpath(argv[0],NULL,NULL,fname,NULL);

        printf("Command line syntax:\n\n");
        printf("    %s <file name> [/Q] [/N] [/Bxx]\n",fname);
        printf("    [] enclosed parameters are optional.\n");
        printf("    /Q enables quiet operation\n");
        printf("    /N disables error messages\n");
        printf("    /Bxx sets buffer size to xx KBytes (Range: 1-63KB)\n");
		return 0;
    }

    if(errorchk(!X12TSR_PRESENT, "X12TSR driver not installed!")) return 0;

    Buffer = (unsigned char *) malloc(BufferSize);
    if(errorchk(Buffer == NULL, "Cannot allocate buffer memory")) return 0;

    regs.x.si =  X12_GETCMDPARAPTR + CARD0;             //Select the driver function
	int86(X12INT,&regs,&regs);							//Call the driver
    Param = (X12PARAM _far *) MK_FP(regs.x.dx, regs.x.ax); //Get the pointer to the driver command structure

    fhan = open(argv[1], O_RDONLY|O_BINARY);
    
    if(errorchk(fhan < 0, "Cannot open file")) 
    	return 0;

    Param->Mode = DECODE;
    regs.x.ax =  X12_VI_START;
    regs.x.si =  X12_SENDCOMMAND + CARD0;               //Select the driver function
	int86(X12INT,&regs,&regs);							//Call the driver

    if(errorchk(regs.x.ax, "Start command timed out"))
    {
   		errorchk(close(fhan) < 0, "Cannot close input file");
    	return 0;
	}
    //read audio and video data

    if(errorchk((buflen = (unsigned int)read(fhan,Buffer,BufferSize)) <= 0, "Insufficient input data")) 
    	return 0;

    OldClock = clock();

    while(1)
    {
        regs.x.ax =  X12_VI_GETSTATUS;
        regs.x.si =  X12_SENDCOMMAND + CARD0;       //Select the driver function
        int86(X12INT,&regs,&regs);                  //Call the driver

        if(kbhit())
        {
            getch();
            break;
        }
        //check digital input zero

        if(Param->DI0 == LOW)
        {
			break;
        }

        if(VideoDone==0)
        {
            if(Param->VQSize != oldVQsizeErr)
            {
                oldVQsizeErr = Param->VQSize;
                OldClock = clock();;
            }
			else
			{
	            if(OldClock > clock())
    	            OldClock = clock();

            	if(errorchk((clock() - OldClock) > CLOCKS_PER_SEC*2, "Decompression failed")) 
                	break;
			}
            //send video data to the 512
            Param->Length = buflen;
            Param->Buffer = Buffer;
            regs.x.ax = X12_VI_PUTDMADATA;
            regs.x.si =  X12_SENDCOMMAND + CARD0;           //Select the "Send command' driver function
            int86(X12INT,&regs,&regs);                      //Call the driver

			if(errorchk(regs.x.ax, "Put DMA data command timed out")) break;

            if(Param->Status == SUCCESS)
            {
                if((buflen = (unsigned int) read(fhan,Buffer,BufferSize)) <= 0)
                {
                    VideoDone = 1;
                }
                if(errorchk((int)buflen == -1, "Problem reading file")) break;
                if(verbose)
                {
                    printf("VQ size=%ld   Digital input=%d\n", Param->VQSize, Param->DI0 & 1);
                }
            }
        }
        else
        {
            //wait for the video queue to empty.
            //if the VQ size is no longer changing, quit.
            if(Param->VQSize == oldVQsize)
            {
                break;
            }
            else
            {
                oldVQsize = Param->VQSize;
                delay(50);
            }
        }
    }

    regs.x.ax =  X12_VI_STOP;
    regs.x.si =  X12_SENDCOMMAND + CARD0;               //Select the driver function
    int86(X12INT,&regs,&regs);                          //Call the driver

	errorchk(close(fhan) < 0, "Cannot close input file");
   
    regs.x.ax =  X12_RESETDEVICE;
    regs.x.si =  X12_SENDCOMMAND + CARD0;               //Select the driver function
    int86(X12INT,&regs,&regs);                          //Call the driver
   
    //flushing keyboard queue

    while(kbhit())
    {
        getch();
    }
	return 0;
}
