#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include "sx12.h"


int errordisp = 1;
int verbose = 1;


int errorchk(int flag, char *errmsg)
{
	if(flag && errordisp)
		printf("Error: %s\n",errmsg);

	return flag;
}

main(int argc, char * argv[])
{
    long            captureframes;
    long            cleanupframes;
    long            BufferSize = 32768L;
    long            maxframes;
    X12PARAM _far   *Param;
    union REGS  	regs;
    int             vihan;
    unsigned char   *Buffer;
    int             a;
    int             firstlow = 1;
    unsigned long   OldFcnt = 0;
    clock_t         OldClock;
    clock_t         maxclock = 0;
    char            fname[10];

    if(argc >= 4)
    {
		captureframes = strtol(argv[2],NULL,10);
		cleanupframes = strtol(argv[3],NULL,10);

        for(a = 4; a < argc; a++)
		{
            if(strnicmp(argv[a],"/q",2)==0)
			{
                verbose = 0; 
			}

            if(strnicmp(argv[a],"/n",2)==0)
			{
                errordisp = 0; 
			}
	        if(strnicmp(argv[a],"/b",2)==0)
			{
	            sscanf(argv[a]+2,"%ld",&BufferSize);
                if(errorchk(BufferSize > 63 || BufferSize < 1, "Buffer size out of range"))
                    return 0;
				BufferSize *= 1024;
			}
		}
    }
    else
    {
        _splitpath(argv[0],NULL,NULL,fname,NULL);
            
        printf("Command line syntax:\n\n");
        printf("    %s <file name> <capture frames> <cleanup frames> [/Q] [/N] [/Bxx]\n",fname);
        printf("    [] enclosed parameters are optional\n");
        printf("    /Q enables quiet operation\n");
        printf("    /N disables error messages\n");
        printf("    /Bxx sets buffer size to xx KBytes (Range: 1-63KB)\n");
		return 0;
    }

    if(errorchk(!X12TSR_PRESENT, "X12TSR driver not installed!")) return 0;

    Buffer = (unsigned char *) malloc(BufferSize);
    if(errorchk(Buffer == NULL, "Cannot allocate buffer memory")) return 0;

    //get the pointer to the command structure

    regs.x.si =  X12_GETCMDPARAPTR + CARD0;     //Select the driver function
    int86(X12INT,&regs,&regs);                  //Call the driver
    Param = (X12PARAM _far *) MK_FP(regs.x.dx, regs.x.ax); //Get the pointer to the driver command structure

    //open the files

    vihan = open(argv[1], O_WRONLY|O_CREAT|O_BINARY|O_TRUNC,S_IRWXU|S_IRWXG|S_IRWXO);

    if(errorchk(vihan < 0, "Cannot open file")) return 0;

    //Start the recording
    Param->DO1   = LOW;
    Param->DDIR1 = OUTPUT;

    Param->Mode = ENCODE;
    regs.x.ax   =  X12_VI_START;
    regs.x.si   =  X12_SENDCOMMAND + CARD0;     //Select the driver function
    int86(X12INT,&regs,&regs);                  //Call the driver

    if(errorchk(regs.x.ax, "Start command timed out")) return 0;

    OldClock = clock();

    regs.x.ax =  X12_VI_GETSTATUS;
    regs.x.si =  X12_SENDCOMMAND + CARD0;       //Select the driver function
    int86(X12INT,&regs,&regs);                  //Call the driver

    maxframes = captureframes + cleanupframes;

    while(Param->FrameCount < maxframes)                    //the sixty adds a delay which gives us time
    {                                           //to empty the video queues after a stop is requested
        if(Param->FrameCount > maxframes - cleanupframes)
        {
            Param->DO1   = HIGH;
            Param->DDIR1 = OUTPUT;
        }

        regs.x.ax =  X12_VI_GETSTATUS;
        regs.x.si =  X12_SENDCOMMAND + CARD0;       //Select the driver function
        int86(X12INT,&regs,&regs);                  //Call the driver

        if(kbhit() && firstlow)
        {
            getch();
            maxframes = Param->FrameCount + cleanupframes;
        }

        //check digital input zero

        if(Param->DI0 == LOW && firstlow)
        {
            maxframes = Param->FrameCount + cleanupframes;
			firstlow = 0;
        }

        //get the current frame count

		if(verbose)
		{
            printf("Frame count=%ld  Digital input=%d Dropped frames=%d\n", Param->FrameCount, Param->DI0 & 1,Param->DroppedFrames);
		}

        if( Param->FrameCount != OldFcnt )
        {
            OldFcnt = Param->FrameCount;
            OldClock = clock();
        }
        else
        {
            if(OldClock > clock())
                OldClock = clock();
            if((clock() - OldClock) > maxclock)
                maxclock = clock() - OldClock;
            if(errorchk((clock() - OldClock) > CLOCKS_PER_SEC*2, "Compression failed")) 
                break;
        }

        //get video data if available

        Param->Buffer = Buffer;
        Param->Length = BufferSize;
        regs.x.ax = X12_VI_GETDMADATA;
        regs.x.si =  X12_SENDCOMMAND + CARD0;           //Select the "Send command' driver function
        int86(X12INT,&regs,&regs);                      //Call the driver

		if(errorchk(regs.x.ax, "Get DMA data command timed out")) break;

        if(Param->Status == SUCCESS)
        {
            if(verbose)
            {
                printf("Frame count=%ld  Digital input=%d Dropped frames=%d\n", Param->FrameCount, Param->DI0 & 1,Param->DroppedFrames);
            }

            if(errorchk((unsigned int)write(vihan,Buffer,Param->Length) < Param->Length, "Cannot write output data"))
				break;
        }
    }
    //stop recording

    regs.x.ax =  X12_VI_STOP;
    regs.x.si =  X12_SENDCOMMAND + CARD0;               //Select the driver function
    int86(X12INT,&regs,&regs);                          //Call the driver

    errorchk(close(vihan) < 0, "Cannot close output file");

    regs.x.ax =  X12_RESETDEVICE;
    regs.x.si =  X12_SENDCOMMAND + CARD0;               //Select the driver function
    int86(X12INT,&regs,&regs);                          //Call the driver

    //printf("%ld\n",maxclock);

    //flushing keyboard queue

    while(kbhit())
    {
        getch();
    }
    return 0;
}

	
